/**
 * Copyright (C) 2008-2012 AgroSense Foundation.
 *
 * AgroSense is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * There are special exceptions to the terms and conditions of the GPLv3 as it is applied to
 * this software, see the FLOSS License Exception
 * <http://www.agrosense.eu/foss-exception.html>.
 *
 * AgroSense is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with AgroSense.  If not, see <http://www.gnu.org/licenses/>.
 */
package nl.cloudfarming.client.geoviewer.jxmap.map.cache;

import java.awt.image.BufferedImage;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import nl.cloudfarming.client.geoviewer.jxmap.map.cache.spi.DefaultTileService;
import org.openide.util.Lookup;

/**
 * Uses lookup to find all implementations of {@link TileService} interface.
 * 
 * <p>
 * {@link DefaultTileService} is added at the top of the list.
 * </p>
 * 
 * @author Frantisek Post
 */
public class TileRepository {

    private static final TileRepository instance = new TileRepository();
    private List<TileService> services;
    
    private TileRepository() {
        services = new ArrayList<>();
        services.add(DefaultTileService.getInstance()); //default cache first
        services.addAll(findImplementations());
    }
    
    private List<TileService> findImplementations() {
        Collection<? extends TileService> caches = Lookup.getDefault().lookupAll(TileService.class);
        return new ArrayList<>(caches);
    }
    
    private TileService getCacheThatContainsTile(TileCacheInfo tileCacheInfo) {
        TileService tileService = null;
        for (TileService service : services) {
            if (service.contains(tileCacheInfo)) {
                tileService = service;
                break;
            }
        }
        return tileService;
    }
    
    //API
    
    public BufferedImage getTile(TileCacheInfo tileCacheInfo) {
        TileService tileService = getCacheThatContainsTile(tileCacheInfo);
        return (tileService != null) ? tileService.getTile(tileCacheInfo) : null;
    } 
    
    public void writeTile(TileCacheInfo tileCacheInfo, BufferedImage image) {
        DefaultTileService.getInstance().writeTile(tileCacheInfo, image);
    }
    
    public static TileRepository getInstance() {
        return instance;
    }
}
